#include "serialport.h"
#include <stdio.h>
#include <stdlib.h>
#include <termios.h>
#include <term.h>
#include <curses.h>
#include <unistd.h>
#include "lcm.h"
#include <pthread.h>

extern struct termios oldtio, newtio;
extern int fd;

int gettingKey = 0;
void* buttonThread(void* arg);

void GP()
{
  int row, column;
  getPositionLcm(&row, &column);
  printf("Cursor is located on row %d, column %d now.\n", row, column);
}

void SP()
{
  int row, column;
  int ret;
  printf("Please input row:[0,1]");
  scanf("%d", &row);
  printf("Please input column:[0-15]");
  scanf("%d", &column);
  ret = setPositionLcm(row, column);
  if(ret < 0)
    printf("Input data incorrect!!\n");
}

void DON()
{
  printf("Turn display on now!\n");
  displayLcm(true);
}

void DOFF()
{
  printf("Turn display off now!\n");
  displayLcm(false);
}

void CON()
{
  printf("Show cursor now!\n");
  cursorLcm(true);
}

void COFF()
{
  printf("Don't show cursor now!\n");
  cursorLcm(false);
}

void CH()
{
  printf("Cursor is moved to home position now!\n");
  cursorActionLcm(HOME);
}

void CMR()
{
  printf("Cursor is moved to next column now!\n");
  cursorActionLcm(MOVERIGHT);
}
    
void CML()
{
  printf("Cursor is moved to previous column now!\n");
  cursorActionLcm(MOVELEFT);
}

void CMB()
{
  printf("Cursor is backspaced now!\n");
  cursorActionLcm(MOVEBACK);
}

void CLR()
{
  printf("LCM is cleared!\n");
  clrscrLcm();
}
struct _FuncTable {
  void (*func)();
  const char str[255];
};

static struct _FuncTable FuncTable[]=
{
  {GP,	  "Get Cursor Position",},
  {SP,	  "Set Cursor Position",},
  {DON,	  "Turn On LCM Display",},
  {DOFF,  "Turn Off LCM Display",},
  {CON,	  "Turn On LCM Cursor",},
  {COFF,  "Turn Off LCM Cursor",},
  {CH,	  "Move Cursor Home",},
  {CMR,	  "Move Cursor Right",},
  {CML,	  "Move Cursor Left",},
  {CMB,	  "Backspace",},
  {CLR,	  "Clear LCM Screen",},
};

int main()
{
  unsigned char msg_demo[] ="Acrosser LCMDemo"; //screen is 16 char wide
  unsigned char msg_bye[] =" Bye Bye ^_^ ";
  unsigned char msg_clr[] ="                ";
  pthread_t th;
  int cmd,i;
  unsigned char input[255];
  clrscrLcm();
  cursorActionLcm(HOME);
  showLcm(16, msg_demo);
	gettingKey = 1;
  pthread_create(&th, NULL, buttonThread, NULL);
  printf("   Acrosser LCM Demo Program\n");
  while(1) {
    printf("\n");
    printf("   ========== MENU =========\n");
    printf("   (0) Exit\n");
    for(i = 0; i < (sizeof(FuncTable) / sizeof(FuncTable[0])); i++) {
      printf("   (%d) ",i+1);
      printf(FuncTable[i].str);
      printf("\n");
    }
    printf("   =========================\n");
CMD_RETRY:
    printf("  >> ");
    scanf("%s",input);
    cmd=atoi(input);
    if ((cmd < 0) || (cmd > ( sizeof(FuncTable) / sizeof(FuncTable[0]) )))
      goto CMD_RETRY;
    if (cmd == 0) {
			gettingKey = 0; 
      break;
		}
    (*(FuncTable[cmd-1].func))();
  }
  setPositionLcm(1, 0);
  showLcm(16, msg_clr);
  setPositionLcm(1, 0);
  showLcm(13, msg_bye);

	pthread_join( th, NULL);
	tcsetattr(fd, TCSANOW, &oldtio);              
  printf("Test finished\n");

	exit(0);
}

void* buttonThread(void* arg)
{
  unsigned char key;
  unsigned char msg_up[] =" Up pushed ";
  unsigned char msg_down[] =" Down pushed ";
  unsigned char msg_left[]=" Left pushed ";
  unsigned char msg_right[] =" Right pushed ";
  unsigned char msg_clr[] ="                ";

  while(gettingKey) {
    usleep(20);
    key = getKeyLcm();
    switch(key) {
      case UP:
        printf("Up button\n");
        setPositionLcm(1, 0);
        showLcm(16, msg_clr);
        setPositionLcm(1, 0);
        showLcm(11, msg_up);
        break;
      case DOWN:
        printf("Down button\n");
        setPositionLcm(1, 0);
        showLcm(16, msg_clr);
        setPositionLcm(1, 0);
        showLcm(13, msg_down);
        break;
      case LEFT:
        printf("Left button\n");
        setPositionLcm(1, 0);
        showLcm(16, msg_clr);
        setPositionLcm(1, 0);
        showLcm(13, msg_left);
        break;
      case RIGHT:
        printf("Right button\n");
        setPositionLcm(1, 0);
        showLcm(16, msg_clr);
        setPositionLcm(1, 0);
        showLcm(14, msg_right);
        break;
      default:
        break;
    }
  }

	pthread_exit(NULL);
}

